/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
import java.awt.geom.Path2D;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.*;
/**
 * The <code>GString</code> class describes the behaviour of the graphic
 * representation of a string in a physic simulator system.
 *
 * @author Omar Munoz, Flavio Silva.
 */
public class GString extends MyString implements Draggable {
   /**
    * X coordinates for the string draw.
    */
   private static final double xPoints[]={0,1.0,1.0,0};
   /**
    * Y coordinates for the string draw.
    */
   private static final double yPoints[]={0,0,2.0,2.0};
   /**
    * Polygon which give a initial form to the string.
    */
   private static final Path2D.Double polyline =
                      new Path2D.Double(Path2D.WIND_EVEN_ODD,xPoints.length);
   /**
    * Final shape of the string, which is drawed.
    */
   private Path2D.Double shape;
   /**
    * Color of the string.
    */
   private Color color = Color.BLACK;
   /**
    * Graphic position of the extreme A of the string.
    */
   private Vector2D graphicsPosA =null;
   /**
    * Graphic position of the extreme B of the string.
    */
   private Vector2D graphicsPosB =null;  // to draw a spring we need
                                      // extremes well definied
                                      // when it is not connected
   /**
    * Generate initial polygon (a rectangle).
    */
   static {  // static initialization block
      polyline.moveTo (xPoints[0], yPoints[0]);
      for (int index = 1; index < xPoints.length;index++)
         polyline.lineTo(xPoints[index], yPoints[index]);
   }
   /**
    * Creates a new <code>GString</code> object, setting its initial position
    * and shape.
    *
    * @param length Length of the string.
    */
   public GString(float length) {
      super(length);
      graphicsPosA = new Vector2D(0,length);
      graphicsPosB = new Vector2D(length,length);
      AffineTransform at = AffineTransform.getTranslateInstance(0,0);
      Vector2D v = getVector();
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(), 1);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
   }
   /**
    * Draw the string, updated by the position of its extremes.
    *
    * @param g Graphic object.
    */
   public void draw (Graphics2D g){
      Vector2D a=getPositionA();
      double ax = a.getX();
      double ay = a.getY();
      Vector2D v = getVector();
      AffineTransform at = AffineTransform.getTranslateInstance(ax, ay);
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(), 1);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      g.setColor(color);
      g.draw(shape);
   }
   /**
    * Determinates if the given coordinates x and y are contained in the string shape.
    *
    * @param x X coordinate.
    * @param y Y coordinate.
    * @return <code>true</code>, if the given coordinates are contained in the shape;
    * <code>false</code> in the opposite case.
    */
   public boolean contains(double x, double y){
      return shape.contains(x,y);
   }
   /**
    * Moves one extreme of the string to the selected coordinates x and y, detaching the
    * block from that extreme, if it is some.
    *
    * @param x X coordinate.
    * @param y Y coordinate.
    */
   public void drag(double x, double y){
      Vector2D a=getPositionA();
      Vector2D b=getPositionB();
      double da = Math.abs(a.getX()-x)+Math.abs(a.getY()-y); // distance to a
      double db = Math.abs(b.getX()-x)+Math.abs(b.getY()-y); // distance to b
      if (da < db) {
         detachBlock_a();
         graphicsPosA.moveTo(x,y);
      }
      else {
         detachBlock_b();
         graphicsPosB.moveTo(x,y);
      }
   }
   /**
    * Gets the position of the extreme A of the string.
    *
    * @return a <code>Vector2D</code> with the position of the block,
    * if there is a block attached to the extreme A;
    * a <code>Vector2D</code> with the graphic position of the extreme A,
    * if there ir no block attached.
    */
   public Vector2D getPositionA() {
      Vector2D p = super.getPositionA();
      if (p != null)
         return p;
      else
         return graphicsPosA;
   }
   /**
    * Gets the position of the extreme B of the string.
    *
    * @return a <code>Vector2D</code> with the position of the block,
    * if there is a block attached to the extreme B;
    * a <code>Vector2D</code> with the graphic position of the extreme B,
    * if there ir no block attached.
    */
   public Vector2D getPositionB() {
      Vector2D p = super.getPositionB();
      if (p != null)
         return p;
      else
         return graphicsPosB;
   }
   /**
    * Update the positions of the extremes of the string.
    */
   public void updateState(){
      if (block_a==null && block_b==null) return;
      if (block_a!=null) {
         Vector2D v = getVector().unitary().times(length);
         graphicsPosB = getPositionA().plus(v);
         return;
      }
      if (block_b!=null) {
         Vector2D v = getVector().unitary().times(length);
         graphicsPosA = getPositionB().minus(v);
         return;
      }

   }
   /**
    * Attachs a block to a extreme of the string, which will be founded in the
    * proximity of the given coordinates x and y.
    *
    * @param x X coordinates.
    * @param y Y coordinates.
    * @param bsc Configuration of elements in the physical system.
    */
   public void attachNearElement(double x, double y, BlockSpringConfiguration bsc){
      Vector2D a=getPositionA();
      if ((a.getX()==x) && (a.getY()==y)) {
         GBlock block = bsc.findBlock(x,y);
         if (block!=null) attachBlock_a(block);
         else
         return;
      }
      Vector2D b=getPositionB();
      if ((b.getX()==x) && (b.getY()==y))  {
         GBlock block = bsc.findBlock(x,y);
         if (block!=null) attachBlock_b(block);
         return;
      }
   }
   /**
    * Sets red color to the string.
    */
   public void setSelectedColor() {
      color = Color.RED;
   }
   /**
    * Sets the default black color to the string.
    */
   public void setDefaultColor() {
      color = Color.BLACK;
   }
}
