/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * This <code>MyString</code> class models the behaviour of a physical string,
 * allowing to attach two blocks to a single string, in order to create a
 * simulated physical system.
 * 
 * @author Omar Munoz, Flavio Silva.
 *
 */
public class MyString extends PhysicsElement{
   /**
    * String identification.
    */
   private static int id=0;
   /**
    * String length.
    */
   protected float length;
   /**
    * String stiffness, activated when the string tries to extend itself more than
    * its length.
    */
   private float stiffness;
   /**
    * Block which going to be attached at the string.
    */
   protected Block block_a, block_b;
   /**
    * Creates a new string.
    * It just has only an identification.
    */
   private MyString(){
      super(id++);
      stiffness=0;
      length=0;
   }
   /**
    * Creates a new string, setting its length.
    *
    * @param length Length of the string.
    */
   public MyString(float length){
      super(id++);
      this.length = length;
      this.stiffness = 0;
      block_a = block_b = null;
   }
   /**
    * Attachs a block to the extreme A of the string.
    *
    * @param a Block which will be attached.
    */
   public void attachBlock_a (Block a) {
      block_a = a;
      block_a.attachString(this);
   }
   /**
    * Detachs the block in the extreme A of the spring.
    */
   public void detachBlock_a(){
      if (block_a == null) return;
      block_a.detachString(this);
      block_a=null;
   }
   /**
    * Attachs a block to the extreme B of the string.
    *
    * @param b Block which will be attached.
    */
   public void attachBlock_b (Block b) {
      block_b = b;
      block_b.attachString(this);
   }
   /**
    * Detachs the block in the extreme B of the spring.
    */
   public void detachBlock_b(){
      if (block_b == null) return;
      block_b.detachString(this);
      block_b=null;
   }
   /**
    * Gets the position of the extreme A of the string.
    *
    * @return a <code>Vector2D</code> with the position of the block,
    * if there is a block attached to the extreme A;
    * <code>null</code>, if there ir no block attached.
    */
   protected Vector2D getPositionA() {
      if (block_a != null)
         return block_a.getPosition();
      return null;
   }
   /**
    * Returns the position of the extreme B of the string.
    *
    * @return a <code>Vector2D</code> with the position of the block,
    * if there is a block attached to the extreme B;
    * <code>null</code>, if there ir no block attached.
    */
   protected Vector2D getPositionB() {
      if (block_b != null)
         return block_b.getPosition();
      return null;
   }
   /**
    * Returns a vector in the direction of the string.
    *
    * @return a <code>Vector2D</code>, which represents a vector in the direction
    * of the string, if one of the positions obtained of extremes A and B are not null;
    * a <code>Vector2D</code> with null length, if positions obtained
    * of extremes A and B are both null;
    *
    */
   protected Vector2D getVector() {
      Vector2D b = getPositionB();
      Vector2D a = getPositionA();
      if (a== null || b==null) return new Vector2D();
      else return b.minus(a);
   }
   /**
    * Returns the force generated by the string.
    * If the string is stretched over its natural length, the stiffness takes
    * a great value (10000), so that the string could return to its natural length;
    * if the string is not stretched (the instantaneous length of the string is
    * shorter than its natural length) the force is zero.
    *
    * @param block The block in which the force will be calculated.
    * @return a <code>Vector2D</code> with the force generated by the string,
    * if the block is attached at the extreme A of the string;
    * a <code>Vector2D</code> with the force generated by the string, but in opposite
    * direction, if the block is attached to the extreme B;
    * a <code>Vector2D</code> with a force equals to zero, if the block is not
    * attached at the string.
    */
   public Vector2D getForce(Block block) {
      Vector2D force;
      if ((block_a == null) || (block_b == null))
         return new Vector2D(); 
      Vector2D stringVector = getVector();
      double stretch = stringVector.module() - length;
      if(stretch > 0){
         stiffness = 1e3f;
         force = stringVector.unitary().times(stretch*stiffness);
      }
      else{
         force = new Vector2D();
      }
      if (block == block_a) return force;
      if (block == block_b) return force.times(-1);
      else return new Vector2D();
   }
   /**
    * Does nothing.
    * @param delta_t Period of simulation.
    */
   public void computeNextState(double delta_t) {
   }
   /**
    * Does nothing.
    */
   public void updateState(){
   }
   /**
    * Sends a description of the string.
    * Sends its identification number and the description of its extremes positions.
    *
    * @return A <code>String</code> describing the <code>MyString</code> object.
    */
   public String getDescription() {
      return "String#"+ getId()+": "+Vector2D.getDescription()+
            ","+Vector2D.getDescription()+",";
   }
   /**
    * Sends the positions of the extremes of the string at a certain moment.
    *
    * @return A <code>String</code> with the positions of the extremes A and B
    * of the string.
    */
   public String getState() {
      String s;
      if (block_a != null) 
         s = block_a.getPosition()+",";
      else if (block_b != null)
              s = block_b.getPosition().minus(new Vector2D(length,0))+",";
            else  
              s = new Vector2D()+",";  // up to here left end is determined 
      if (block_a != null)
         s += block_a.getPosition();
      else if (block_b != null)
               s += block_b.getPosition().plus(new Vector2D(length,0));
            else
               s += new Vector2D();
      return s+",";
   }
    
}
